<?php
/**
 * User: nester_a
 * Date: 30.06.14
 */

namespace LMS\Libraries\Base;

use InvalidArgumentException;
use JFactory;
use JLog;
use JPath;
use JText;
use LMS\Libraries\ServiceLocator;
use UnexpectedValueException;

class Table extends \JTable
{

	const MANY = 1;
	const ONE = 0;

	/**
	 * This array defines relations between tables
	 * Should be defined in Table constructor
	 * format:
	 * array(
	 *   'relation_name' => new \LMS\{Some}\{Path}\{SomeTable}
	 *   'relation_name2' => new \LMS\{Some}\{Path}\{SomeTable2}
	 * )
	 *
	 * @var array
	 */
	protected $relations = array();

	/**
	 * Array used to map one field name to another
	 * For example map `id` field to 'uid' field
	 *
	 * @var array
	 */
	protected $fieldMap = array();

	protected $alias = null;

	private static $instance = array();

	public function __construct($table, $key, $db = false)
	{		
		parent::__construct($table, $key, $db ? $db : ServiceLocator::get('db'));
	}

	public function bind($src, $ignore = array())
	{
		if (!is_object($src) && !is_array($src)){
			throw new InvalidArgumentException(sprintf('%s::bind(*%s*)', get_class($this), gettype($src)));
		}

		$src = (array)$src;
		foreach($src as $key=>$value){
			if(isset($this->fieldMap[$key])){
				$src[$this->fieldMap[$key]] = $value;
			}
		}		

		return parent::bind($src, $ignore); // TODO: Change the autogenerated stub
	}


	/**
	 * @param bool $withIDfield
	 * @return array
	 */
	public function getColumns($withIDfield = false)
	{
		$keys = array_keys(getPublicObjectVars($this));
		$columns = array();
		foreach($keys as $key){
			if($key != $this->getKeyName() || $withIDfield){
				$columns[] = $this->getAlias($key);
			}
		}

		return $columns;
	}

	public function getRealColumns($withIDfield = false)
	{
		$keys = array_keys(getPublicObjectVars($this));
		$columns = array();
		foreach($keys as $key){
			if($key != $this->getKeyName() || $withIDfield){
				$columns[] = $key;
			}
		}

		return $columns;
	}

	/**
	 * @param bool $withIDfield
	 * @return array
	 */
	public function getValues($withIDfield = false)
	{
		$columns = $this->getColumns($withIDfield);
		$values = array();
		foreach($columns as $column){
			$realColumnName = $this->getField($column);
			$values[$realColumnName] = $this->getDbo()->quote($this->$realColumnName);
		}

		return $values;
	}

	/**
	 * @param string $name
	 * @return Table
	 * @throws \InvalidArgumentException
	 */
	public function getRelation($name)
	{
		if(empty($this->relations)){
			$this->initRelations();
		}

		if(!isset($this->relations[$name])){
			throw new \InvalidArgumentException('There is no table relations with name "'.$name.'"');
		}
		return $this->relations[$name];
	}

	/**
	 * Return field name
	 * Used mostly when building queries
	 *
	 * $name can be 'fieldname' or 'relationtable.fieldname'
	 *
	 * @param string $name
	 * @return string
	 */
	public function getField($name)
	{
		$explodedName = explode('.', $name);
		if(count($explodedName) == 1){
			return isset($this->fieldMap[$name]) ? $this->fieldMap[$name] : $name;
		}else{
			return $this->getRelation($explodedName[0])->getField($explodedName[1]);
		}
	}

	/**
	 * Return alias for field name
	 * Used mostly when building queries
	 *
	 * $name can be 'fieldname' or 'relationtable.fieldname'
	 *
	 * @param string $name
	 * @return string
	 */
	public function getAlias($name)
	{
		$explodedName = explode('.', $name);
		if(count($explodedName) == 1){
			foreach($this->fieldMap as $alias => $realName){
				if($realName == $name){
					return $alias;
				}
			}
			return $name;
		}else{
			return $this->getRelation($explodedName[0])->getAlias($explodedName[1]);
		}
	}

	/**
	 * Return existing or generate table alias if not exist
	 *
	 * @return null|string
	 */
	public function getTableAlias()
	{
		if(!is_null($this->alias)){
			return $this->alias;
		}

		return substr(md5($this->getTableName()), 2, 7);

	}

	/**
	 * @param $alias
	 */
	public function setTableAlias($alias)
	{
		$this->alias = $alias;
	}

	/**
	 * @return \LMS\Libraries\Database\Driver
	 */
	public function getDbo()
	{
		return parent::getDbo();
	}

	/**
	 * @return array
	 */
	public function getRelations()
	{
		if(empty($this->relations)){
			$this->initRelations();
		}
		return $this->relations;
	}

	/*
	 * Initialise relationships
	 */
	protected function initRelations()
	{
		$this->relations = array();
	}

	/*
	 * Add relationship
	 */
	protected function addRelation($name, $ralation)
	{
		$name = strtolower($name);
		$this->relations[$name] = $ralation;
	}

}

function getPublicObjectVars($obj) {
	return get_object_vars($obj);
}